##############################################
# Display error and close procedure 
##############################################

# Display error and close procedure
proc DisplayErrorAndClose {socketID code APIName} {
    global tcl_argv
    if {$code != -2} {
        set code2 [catch "ErrorStringGet $socketID $code strError"]
        if {$code2 != 0} {
            puts "$APIName ERROR => $code - ErrorStringGet ERROR => $code2"
            set tcl_argv(0) "$APIName ERROR => $code"
        } else {
            puts stdout "$APIName ERROR => $code : $strError"
            set tcl_argv(0) "$APIName ERROR => $code : $strError"
        }
    } else {
        puts stdout "$APIName ERROR => $code : TCP timeout"
        set tcl_argv(0) "$APIName ERROR => $code : TCP timeout"
    }
    set code2 [catch "TCP_CloseSocket $socketID"] 
    return
}

##################################################
# Main process
##################################################

# Initialization
set TimeOut 10
set group "alignstation"
set axis1 "alignstation.middle"
set axis2 "alignstation.base"
set analogin "GPIO2.ADC1"

# Open TCP socket
set code [catch "OpenConnection $TimeOut socketID"]
if {$code != 0} {
	puts stdout "OpenConnection failed => $code" 
} else {

    # Kill group
    set code [catch "GroupKill $socketID $group"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupKill" 
        return
    }

    # Initialize group
    set code [catch "GroupInitialize $socketID $group"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupInitialize" 
        return
    }

    # Home group
    set code [catch "GroupHomeSearch $socketID $group"]
    if {$code != 0} {     
        DisplayErrorAndClose $socketID $code "GroupHomeSearch" 
        return
    }

    # Move group with 5 relative units
	for { set var 0 } { $var <= 5 } { incr var } {
        set code [catch "GroupMoveRelative $socketID $group 1 1"]
        if {$code != 0} {	
            DisplayErrorAndClose $socketID $code "GroupMoveRelative" 
            return
        }
    }

    # Get analog value
    set code [catch "GPIOAnalogGet $socketID $analogin voltage"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GPIOAnalogGet" 
        return
    }

    # Test if voltage is greater than 0.2 volt
    while { $voltage >= 0.2 } { 

        # Get analog value 
        set code [catch "GPIOAnalogGet $socketID $analogin voltage"]
        if {$code != 0} {
            DisplayErrorAndClose $socketID $code "GPIOAnalogGet" 
            return
        }
        
        set move1 $voltage
        set move2 [expr { $voltage * -1 } ]
        puts stdout "$analogin: $voltage volt(s)"
        puts stdout "           move axis1: $move1"
        puts stdout "           move axis2: $move2"
    
        # Move axis 1
        set code [catch "GroupMoveAbsolute $socketID $axis1 $move1"]
        if {$code != 0} {
            DisplayErrorAndClose $socketID $code "GroupMoveAbsolute" 
            return
        }
    
        # Move axis 2
        set code [catch "GroupMoveAbsolute $socketID $axis2 $move2"]
        if {$code != 0} {
            DisplayErrorAndClose $socketID $code "GroupMoveAbsolute" 
            return
        }

    }

    # Wait 1 second and close socket
    after 1000	
    puts "End of program"
    TCP_CloseSocket $socketID
}
