##############################################
# Display error and close procedure 
##############################################

# Display error and close procedure
proc DisplayErrorAndClose {socketID code APIName} {
    global tcl_argv
    if {$code != -2} {
        set code2 [catch "ErrorStringGet $socketID $code strError"]
        if {$code2 != 0} {
            puts "$APIName ERROR => $code - ErrorStringGet ERROR => $code2"
            set tcl_argv(0) "$APIName ERROR => $code"
        } else {
            puts stdout "$APIName ERROR => $code : $strError"
            set tcl_argv(0) "$APIName ERROR => $code : $strError"
        }
    } else {
        puts stdout "$APIName ERROR => $code : TCP timeout"
        set tcl_argv(0) "$APIName ERROR => $code : TCP timeout"
    }
    set code2 [catch "TCP_CloseSocket $socketID"] 
    return
}


##################################################
# Main process
##################################################

# Initialization
set TimeOut 60
set Group "SINGLE_AXIS"
set Positioner "SINGLE_AXIS.MY_STAGE"
set Type1 "SINGLE_AXIS.MY_STAGE.ExternalLatchPosition"
set Type2 "GPIO2.ADC1"
set Event "Immediate"
set Action "ExternalGatheringRun"
set NbPoints 20
set Div 1
set Current 0
set code 0

# Open TCP socket
set code [catch "OpenConnection $TimeOut socketID"]
if {$code != 0} {
    puts stdout "OpenConnection failed => $code" 
} else {

    # Kill group    
    set code [catch "GroupKill $socketID $Group"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupKill" 
        return
    }

    # Initialize group
    set code [catch "GroupInitialize $socketID $Group"]
    if {$code != 0} {         	   
        DisplayErrorAndClose $socketID $code "GroupInitialize" 
        return
    }

    # Home group
    set code [catch "GroupHomeSearch $socketID $Group"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupHomeSearch" 
        return
    }

    # Configure gathering parameters
    set code [catch "GatheringExternalConfigurationSet $socketID $Type1 $Type2"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GatheringExternalConfigurationSet" 
        return
    }

    # Add an event
    set code [catch "EventAdd $socketID $Positioner $Event 0 $Action $NbPoints $Div 0"]  
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "EventAdd" 
        return
    }

    # Push on TRIG IN button

    puts "External gathering"

    # Wait end of external gathering
    while {$Current < $NbPoints} {
        # Get current acquired point number 
        set code [catch "GatheringExternalCurrentNumberGet $socketID Current Max"]
        if {$code != 0} {
            DisplayErrorAndClose $socketID $code "GatheringExternalCurrentNumberGet" 
            return
        } else {
            puts stdout "current number: $Current"
            after 1000
        }
    }

    # Stop external gathering and save data
    set code [catch "GatheringExternalStopAndSave $socketID"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GatheringExternalStopAndSave" 
        return
    }

    # Close TCP socket
    set code [catch "TCP_CloseSocket $socketID"]
}
