##############################################
# Display error and close procedure 
##############################################

# Display error and close procedure
proc DisplayErrorAndClose {socketID code APIName} {
    global tcl_argv
    if {$code != -2} {
        set code2 [catch "ErrorStringGet $socketID $code strError"]
        if {$code2 != 0} {
            puts "$APIName ERROR => $code - ErrorStringGet ERROR => $code2"
            set tcl_argv(0) "$APIName ERROR => $code"
        } else {
            puts stdout "$APIName ERROR => $code : $strError"
            set tcl_argv(0) "$APIName ERROR => $code : $strError"
        }
    } else {
        puts stdout "$APIName ERROR => $code : TCP timeout"
        set tcl_argv(0) "$APIName ERROR => $code : TCP timeout"
    }
    set code2 [catch "TCP_CloseSocket $socketID"] 
    return
}


##################################################
# Main process
##################################################

# Initialization
set TimeOut 60
set Group "SINGLE_AXIS"
set Positioner "SINGLE_AXIS.MY_STAGE"
set Type1 "SINGLE_AXIS.MY_STAGE.SetpointPosition"
set Type2 "SINGLE_AXIS.MY_STAGE.CurrentPosition"
set Event "SGamma.MotionStart"
set Action "GatheringRun"
set Displacement 50
set NbPoints 1000
set Div 100	
set code 0

# Open TCP socket
set code [catch "OpenConnection $TimeOut socketID"]
if {$code != 0} {
    puts stdout "OpenConnection failed => $code" 
} else {

    # Kill group    
    set code [catch "GroupKill $socketID $Group"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupKill" 
        return
    }

    # Initialize group
    set code [catch "GroupInitialize $socketID $Group"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupInitialize" 
        return
    }

    # Home group
    set code [catch "GroupHomeSearch $socketID $Group"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupHomeSearch" 
        return
    }

    # Configure gathering parameters
    set code [catch "GatheringConfigurationSet $socketID $Type1 $Type2"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GatheringConfigurationSet" 
        return
    }

    # Add an event
    set code [catch "EventAdd $socketID $Positioner $Event 0 $Action $NbPoints $Div 0"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "EventAdd" 
        return
    }

    # Move positioner
    set code [catch "GroupMoveRelative $socketID $Group $Displacement"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupMoveRelative" 
        return
    }

    # Stop gathering and save data
    set code [catch "GatheringStopAndSave $socketID"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GatheringStopAndSave" 
        return
    }

    # Close TCP socket
    set code [catch "TCP_CloseSocket $socketID"]
}
