##############################################
# Display error and close procedure 
##############################################

# Display error and close procedure
proc DisplayErrorAndClose {socketID code APIName} {
    global tcl_argv
    if {$code != -2} {
        set code2 [catch "ErrorStringGet $socketID $code strError"]
        if {$code2 != 0} {
            puts "$APIName ERROR => $code - ErrorStringGet ERROR => $code2"
            set tcl_argv(0) "$APIName ERROR => $code"
        } else {
            puts stdout "$APIName ERROR => $code : $strError"
            set tcl_argv(0) "$APIName ERROR => $code : $strError"
        }
    } else {
        puts stdout "$APIName ERROR => $code : TCP timeout"
        set tcl_argv(0) "$APIName ERROR => $code : TCP timeout"
    }
    set code2 [catch "TCP_CloseSocket $socketID"] 
    return
}


##################################################
# Main process
##################################################

# Initialization
set TimeOut 60
set Group "XY"
set Positioner "XY.X"
set Velocity1 20
set Velocity2 -30
set Acceleration 80
set code 0

# Open TCP socket
set code [catch "OpenConnection $TimeOut socketID"]
if {$code != 0} {
    puts stdout "OpenConnection failed => $code" 
} else {

    # Kill group    
    set code [catch "GroupKill $socketID $Group"]
    if {$code != 0} {
   	    DisplayErrorAndClose $socketID $code "GroupKill" 
        return
    }

    # Initialize group
    set code [catch "GroupInitialize $socketID $Group"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupInitialize" 
        return
    }

    # Home group
    set code [catch "GroupHomeSearch $socketID $Group"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupHomeSearch" 
        return
    }

    # Enable jog mode (group must be ready)
    set code [catch "GroupJogModeEnable $socketID $Group"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupJogModeEnable" 
        return
    }

    # Set jog parameters to move a positioner => constant velocity is not null
    set code [catch "GroupJogParametersSet $socketID $Positioner $Velocity1 $Acceleration"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupJogParametersSet" 
        return
    }

    # Wait 3 seconds
    after 3000

    # Set jog parameters to move the positioner in the reverse sense => constant velocity is not null
    set code [catch "GroupJogParametersSet $socketID $Positioner $Velocity2 $Acceleration"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupJogParametersSet" 
        return
    }

    # Wait 3 seconds
    after 3000

    # Set jog parameters to stop a positioner => constant velocity is null
    set code [catch "GroupJogParametersSet $socketID $Positioner 0 $Acceleration"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupJogParametersSet" 
        return
    }

    # Disable jog mode (constant velocity must be null on all positioners from group)
    set code [catch "GroupJogModeDisable $socketID $Group"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupJogModeDisable" 
        return
    }

    # Close TCP socket
    set code [catch "TCP_CloseSocket $socketID"]
}
