##############################################
# Display error and close procedure 
##############################################

# Display error and close procedure
proc DisplayErrorAndClose {socketID code APIName} {
    global tcl_argv
    if {$code != -2} {
        set code2 [catch "ErrorStringGet $socketID $code strError"]
        if {$code2 != 0} {
            puts "$APIName ERROR => $code - ErrorStringGet ERROR => $code2"
            set tcl_argv(0) "$APIName ERROR => $code"
        } else {
            puts stdout "$APIName ERROR => $code : $strError"
            set tcl_argv(0) "$APIName ERROR => $code : $strError"
        }
    } else {
        puts stdout "$APIName ERROR => $code : TCP timeout"
        set tcl_argv(0) "$APIName ERROR => $code : TCP timeout"
    }
    set code2 [catch "TCP_CloseSocket $socketID"] 
    return
}


##################################################
# Main process
##################################################

# Initialization
set TimeOut 120
set Moteur "XY"
set Mot "XY.X_VP"
set A "XY.X_VP.SetpointPosition"
set B "XY.X_VP.CurrentPosition"
set C "XY.X_VP.SetpointVelocity"
set D "XY.X_VP.SetpointAcceleration"
set Event "Immediate"
set Action "GatheringRun"
set Num 0

# Open TCP socket
set code [catch "OpenConnection $TimeOut socketID"]
if {$code != 0} {
    puts stdout "OpenConnection failed => $code" 
} else {

    # Kill group
    set code [catch "GroupKill $socketID $Moteur"]
    if {$code != 0} {
     	DisplayErrorAndClose $socketID $code "GroupKill" 
        return
    }

    # Initialize group
    set code [catch "GroupInitialize $socketID $Moteur"]		
   	if {$code != 0} {
       	DisplayErrorAndClose $socketID $code "GroupInitialize" 
        return
    }

    # Home group
    set code [catch "GroupHomeSearch $socketID $Moteur"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupHomeSearch" 
        return
    }

    # Set gathering parameters	
    set code [catch "GatheringConfigurationSet $socketID $A $B $C $D"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GatheringConfigurationSet" 
        return
    }

    # Get gathering parameters
    set code [catch "GatheringConfigurationGet $socketID J"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GatheringConfigurationGet" 
        return
    } else {
   		puts stdout "Data types to be gathered: $J"
    }	
   
    # Get gathering current acquired point number
    set code [catch "GatheringCurrentNumberGet $socketID Num Max"]
    if {$code != 0} {
       	DisplayErrorAndClose $socketID $code "GatheringCurrentNumberGet" 
        return
    } else {
 		puts stdout "Maximum possible number of acquisition per type of data: $Max"
    }

    # Add an event
    set code [catch "EventAdd $socketID $Mot $Event 0 $Action 20000 10 0"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "EventAdd" 
        return
    }

    # Enable jog mode
    set code [catch "GroupJogModeEnable $socketID $Moteur"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupJogModeEnable" 
        return
    }

    # Wait 2 seconds
    after 2000

    puts "Jog moves and data acquisition"
    # Set jog parameters to move both positioners in the positive direction 
    set code [catch "GroupJogParametersSet $socketID $Moteur 5 50 5 50"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupJogParametersSet" 
        return
    }
    puts "  X and Y positioners going in positive direction during 500 msec"
    puts "      X and Y positioners speed = 5 / Acceleration = 50"

    # Wait 500 milliseconds
    after 500
    # Set jog parameters to move both positioners, the first in the positive direction, the second in the negative
    set code [catch "GroupJogParametersSet $socketID $Moteur 10 50 -10 50"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupJogParametersSet" 
        return
    }
    puts "  X positioner going in positive direction during 2 sec"
    puts "  Y positioner going in negative direction during 2 sec"
    puts "      X positioner speed = 10 / Acceleration = 50"
    puts "      Y positioner speed = -10 / Acceleration = 50"

    # Wait 2 seconds
    after 2000
    # Set jog parameters to move both positioners in the reverse sense 
    set code [catch "GroupJogParametersSet $socketID $Moteur -10 50 20 50"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupJogParametersSet" 
        return
    }
    puts "  X positioner going in negative direction during 2 sec"
    puts "  Y positioner going in positive direction during 2 sec"
    puts "      X positioner speed = -10 / Acceleration = 50"
    puts "      Y positioner speed = 20 / Acceleration = 50"

    # Wait 2 seconds
    after 2000
    # Set jog parameters to move both positioners in the negative direction
    set code [catch "GroupJogParametersSet $socketID $Moteur -5 50 -5 50"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupJogParametersSet" 
        return
    }
    puts "  X and Y positioners going in negative direction during 500 msec"
    puts "      X and positioner speed = -5 / Acceleration = 50"

    # Wait 500 milliseconds
    after 500
	# Set jog parameters to stop the positioners => constant velocities are null
    set code [catch "GroupJogParametersSet $socketID $Moteur 0 50 0 50"]
    if {$code != 0} {	
        DisplayErrorAndClose $socketID $code "GroupJogParametersSet" 
        return
    }
    puts "  X and Y positioners stopped"
    puts "      X and Y positioner speed = 0 / Acceleration = 50"

    # Wait 500 milliseconds
    after 500

    # Disable jog mode
    set code [catch "GroupJogModeDisable $socketID $Moteur"]
    if {$code != 0} {	
        DisplayErrorAndClose $socketID $code "GroupJogModeDisable" 
        return
    }

    # Stop gathering and save data
    set code [catch "GatheringStopAndSave $socketID"]
    if {$code != 0} {	
        DisplayErrorAndClose $socketID $code "GatheringStopAndSave" 
        return
    }

    # Close socket
    puts "End of program"
    TCP_CloseSocket $socketID
} 
