##############################################
# Display error and close procedure 
##############################################

# Display error and close procedure
proc DisplayErrorAndClose {socketID code APIName} {
    global tcl_argv
    if {$code != -2} {
        set code2 [catch "ErrorStringGet $socketID $code strError"]
        if {$code2 != 0} {
            puts "$APIName ERROR => $code - ErrorStringGet ERROR => $code2"
            set tcl_argv(0) "$APIName ERROR => $code"
        } else {
            puts stdout "$APIName ERROR => $code : $strError"
            set tcl_argv(0) "$APIName ERROR => $code : $strError"
        }
    } else {
        puts stdout "$APIName ERROR => $code : TCP timeout"
        set tcl_argv(0) "$APIName ERROR => $code : TCP timeout"
    }
    set code2 [catch "TCP_CloseSocket $socketID"] 
    return
}


##################################################
# Main process
##################################################

# Initialization
set TimeOut 60
set Group "SINGLE_AXIS"
set Positioner "SINGLE_AXIS.MY_STAGE"
set StartPosition -15
set Displacement 25
set MinPos -10
set MaxPos 10
set StepPos 1
set code 0

# Open TCP socket
set code [catch "OpenConnection $TimeOut socketID"]
if {$code != 0} {
    puts stdout "OpenConnection failed => $code" 
} else {

    # Kill group    
    set code [catch "GroupKill $socketID $Group"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupKill" 
        return
    }

    # Initialize group
    set code [catch "GroupInitialize $socketID $Group"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupInitialize" 
        return
    }

    # Home group
    set code [catch "GroupHomeSearch $socketID $Group"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupHomeSearch" 
        return
    }

    # Move positioner to start position
    set code [catch "GroupMoveAbsolute $socketID $Group $StartPosition"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupMoveAbsolute" 
        return
    }

    # Set position compare parameters
    set code [catch "PositionerPositionCompareSet $socketID $Positioner $MinPos $MaxPos $StepPos"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "PositionerPositionCompareSet" 
        return
    }

    # Enable position compare mode
    set code [catch "PositionerPositionCompareEnable $socketID $Positioner"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "PositionerPositionCompareEnable" 
        return
    }

    # Move positioner
    set code [catch "GroupMoveRelative $socketID $Group $Displacement"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupMoveRelative" 
        return
    }

    # Disable position compare mode
    set code [catch "PositionerPositionCompareDisable $socketID $Positioner"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "PositionerPositionCompareDisable" 
        return
    }

    # Close TCP socket
    set code [catch "TCP_CloseSocket $socketID"]
}
