##############################################
# Display error and close procedure 
##############################################

# Display error and close procedure
proc DisplayErrorAndClose {socketID code APIName} {
    global tcl_argv
    if {$code != -2} {
        set code2 [catch "ErrorStringGet $socketID $code strError"]
        if {$code2 != 0} {
            puts "$APIName ERROR => $code - ErrorStringGet ERROR => $code2"
            set tcl_argv(0) "$APIName ERROR => $code"
        } else {
            puts stdout "$APIName ERROR => $code : $strError"
            set tcl_argv(0) "$APIName ERROR => $code : $strError"
        }
    } else {
        puts stdout "$APIName ERROR => $code : TCP timeout"
        set tcl_argv(0) "$APIName ERROR => $code : TCP timeout"
    }
    set code2 [catch "TCP_CloseSocket $socketID"] 
    return
}



##################################################
# Main process
##################################################

# Initialization
set TCPTimeOut 0.5
set code 0
set ISRPeriodSec 0.0001
set Positioner "SINGLE_AXIS.MY_STAGE"
set TimerName "Timer1"
set TimerPeriodSec 2
set EvtParam 0
set Action "ExecuteTCLScript"
set TCLFile "12-2-MyScript.tcl"
set TCLTask "MyTask"
set TCLArgs "0"
set GlobalVarNumber 1
set Value 5

# Open TCP socket 
set code [catch "OpenConnection $TCPTimeOut socketID"]
if {$code != 0} {
    puts stdout "OpenConnection failed => $code" 
} else {

    # Calculate divisor (periods are in seconds)
    set Divisor [expr {$TimerPeriodSec / $ISRPeriodSec}]
    puts stdout "Divisor: $Divisor"

    set Divisor [expr {int($Divisor)} ]
    puts stdout "Divisor troncated in integer: $Divisor"

    # Configure a timer 
    set code [catch "TimerSet $socketID $TimerName $Divisor"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "TimerSet" 
        return
    } else {
        puts "Timer set"
    }

    # Add timer event with an action that allows to execute MyScript.tcl
    set code [catch "EventAdd $socketID $Positioner $TimerName $EvtParam $Action $TCLFile $TCLTask $TCLArgs"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "EventAdd" 
        return
    }

    # Set global variable
    set code [catch "GlobalArraySet $socketID $GlobalVarNumber $Value"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GlobalArraySet" 
        return
    }

    # close TCP socket
    set code [catch "TCP_CloseSocket $socketID"]
}
