##############################################
# Display error and close procedure 
##############################################

# Display error and close procedure
proc DisplayErrorAndClose {socketID code APIName} {
    global tcl_argv
    if {$code != -2} {
        set code2 [catch "ErrorStringGet $socketID $code strError"]
        if {$code2 != 0} {
            puts "$APIName ERROR => $code - ErrorStringGet ERROR => $code2"
            set tcl_argv(0) "$APIName ERROR => $code"
        } else {
            puts stdout "$APIName ERROR => $code : $strError"
            set tcl_argv(0) "$APIName ERROR => $code : $strError"
        }
    } else {
        puts stdout "$APIName ERROR => $code : TCP timeout"
        set tcl_argv(0) "$APIName ERROR => $code : TCP timeout"
    }
    set code2 [catch "TCP_CloseSocket $socketID"] 
    return
}



##################################################
# Main process
##################################################

# Initialization
set TimeOut 20
set code 0
set group "SINGLE_AXIS"

# Open TCP socket
set code [catch "OpenConnection $TimeOut socketID"]
if {$code != 0} {
    puts stdout "OpenConnection failed => $code" 
} else {

    # Recover the input arguments entered by the user
    if {$tcl_argc == 3} {
        set startpos $tcl_argv(0)
      	set endpos $tcl_argv(1)
      	set cycles $tcl_argv(2)
    } else {
      	puts stdout "Wrong number of parameters, 3 are needed"
      	set code [catch "TCP_CloseSocket $socketID"]
      	return
    }

    # Kill group 
    set code [catch "GroupKill $socketID $group"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupKill" 
        return 
    }

    # Initialize group
    set code [catch "GroupInitialize $socketID $group"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupInitialize" 
 		return 
    }

    # Home group
    set code [catch "GroupHomeSearch $socketID $group"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupHomeSearch" 
 		return 
    }

    # Loop until the number of cycles (third parameter entered by the user) is reached
    for { set i 0} {($i < $cycles) } {incr i} {

        # Move group to start position 
        set code [catch "GroupMoveAbsolute $socketID $group $startpos"]
        if {$code != 0} {
            DisplayErrorAndClose $socketID $code "GroupMoveAbsolute" 
            return            
        }

        # Move group to end position
        set code [catch "GroupMoveAbsolute $socketID $group $endpos"]
        if {$code != 0} {
            DisplayErrorAndClose $socketID $code "GroupMoveAbsolute" 
            return            
        }
    }

    # Close TCP socket
    set code [catch "TCP_CloseSocket $socketID"]
}
